"use strict";

var _deriveName = _interopRequireDefault(require("./deriveName"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

module.exports = ({
  types: t
}) => {
  const replace = (path, name, replacement) => {
    const id = t.identifier(name);
    path.replaceWithMultiple([t.variableDeclaration('const', [t.variableDeclarator(id, replacement)]), t.exportDefaultDeclaration(id)]);
  };

  return {
    visitor: {
      ExportDefaultDeclaration(path, state) {
        const declaration = path.node.declaration;

        if (declaration.id && declaration.id.name) {
          return;
        }

        const name = (0, _deriveName.default)(state, path.scope);

        if (t.isArrowFunctionExpression(declaration)) {
          const declarationReplacement = t.arrowFunctionExpression(declaration.params, declaration.body, declaration.generator);
          declarationReplacement.async = declaration.async;
          replace(path, name, declarationReplacement);
          return;
        }

        if (t.isFunctionDeclaration(declaration)) {
          const declarationReplacement = t.functionExpression(null, declaration.params, declaration.body, declaration.generator);
          declarationReplacement.async = declaration.async;
          replace(path, name, declarationReplacement);
          return;
        }

        if (t.isFunctionExpression(declaration)) {
          const declarationReplacement = t.functionExpression(null, declaration.params, declaration.body, declaration.generator);
          declarationReplacement.async = declaration.async;
          replace(path, name, declarationReplacement);
          return;
        }

        if (t.isClassDeclaration(declaration)) {
          const declarationReplacement = t.classExpression(null, declaration.superClass, declaration.body, declaration.decorators || []);
          replace(path, name, declarationReplacement); // eslint-disable-next-line no-useless-return

          return;
        }
      }

    }
  };
};